/* 
   ipmi-serial-cmds.c - IPMI serial port settings commands

   Copyright (C) 2003 FreeIPMI Core Team

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>

#ifdef STDC_HEADERS
#include <string.h>
#endif

#include <errno.h>

/* #define DEBUG */

#include "freeipmi.h"

fiid_template_t tmpl_set_serial_conf_param_rs =
  {
    {8, "cmd"},
    {8, "comp_code"},
    {0, ""}
  };

fiid_template_t tmpl_get_serial_conf_param_rq =
  {
    {8, "cmd"},

    {4, "channel_number"},
    {3, "reserved"},
    {1, "parameter_type"},

    {8, "parameter_selector"},
    {8, "set_selector"},
    {8, "block_selector"},

    {0, ""}
  };

fiid_template_t tmpl_set_serial_conf_param_connmode_rq =
  {
    {8, "cmd"},
    
    {4, "channel_number"},
    {4, "reserved"},
    
    {8, "parameter_selector"}, 
    
    {1, "basic_mode_enable"},
    {1, "ppp_mode_enable"},
    {1, "terminal_mode_enable"},
    {4, "reserved2"},
    {1, "direct"},
    
    {0, ""}
  };

fiid_template_t tmpl_get_serial_conf_param_connmode_rs =
  {
    {8, "cmd"},

    {8, "comp_code"},

    {4, "present_revision"},
    {4, "oldest_revision_parameter"},

    {1, "basic_mode_enable"},
    {1, "ppp_mode_enable"},
    {1, "terminal_mode_enable"},
    {4, "reserved2"},
    {1, "direct"},

    {0, ""}
  };

fiid_template_t tmpl_set_serial_conf_param_pageblackout_rq =
  {
    {8, "cmd"},

    {4, "channel_number"},
    {4, "reserved"},
    
    {8, "parameter_selector"}, 
    
    {8, "page_blackout_interval"},

    {0, ""}
  };

fiid_template_t tmpl_get_serial_conf_param_pageblackout_rs =
  {
    {8, "cmd"},

    {8, "comp_code"},

    {4, "present_revision"},
    {4, "oldest_revision_parameter"},

    {8, "page_blackout_interval"},

    {0, ""}
  };

fiid_template_t tmpl_set_serial_conf_param_retry_rq =
  {
    {8, "cmd"},
    
    {4, "channel_number"},
    {4, "reserved"},
    
    {8, "parameter_selector"}, 
    
    {8, "retry_time"},
    
    {0, ""}
  };

fiid_template_t tmpl_get_serial_conf_param_retry_rs =
  {
    {8, "cmd"},

    {8, "comp_code"},

    {4, "present_revision"},
    {4, "oldest_revision_parameter"},

    {8, "retry_time"},

    {0, ""}
  };

fiid_template_t tmpl_set_serial_conf_param_commbits_rq =
  {
    {8, "cmd"},
    
    {4, "channel_number"},
    {4, "reserved"},
    
    {8, "parameter_selector"}, 
    
    {5, "reserved2"},
    {1, "dtr_hangup"},
    {2, "flow_control"},
    
    {4, "bit_rate"},
    {4, "reserved3"},
    
    {0, ""}
  };
    
fiid_template_t tmpl_get_serial_conf_param_commbits_rs =
  {
    {8, "cmd"},

    {8, "comp_code"},

    {4, "present_revision"},
    {4, "oldest_revision_parameter"},

    {5, "reserved2"},
    {1, "dtr_hangup"},
    {2, "flow_control"},

    {4, "bit_rate"},
    {4, "reserved3"},

    {0, ""}
  };

int 
fill_set_serial_connmode (fiid_obj_t obj_data_rq, 
			  u_int8_t channel_number, 
			  u_int8_t basic_mode_enable,
			  u_int8_t ppp_mode_enable,
			  u_int8_t terminal_mode_enable,
			  u_int8_t direct)
{
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_connmode_rq, 
		"cmd", 
		IPMI_CMD_SET_SERIAL_MODEM_CONF_PARAM);
    
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_connmode_rq, 
		"channel_number", 
		channel_number);
  
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_connmode_rq, 
		"parameter_selector", 
		IPMI_SERIAL_PARAM_CONNECTION_MODE);
  
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_connmode_rq, 
		"basic_mode_enable", 
		basic_mode_enable);
  
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_connmode_rq, 
		"ppp_mode_enable", 
		ppp_mode_enable);
  
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_connmode_rq, 
		"terminal_mode_enable", 
		terminal_mode_enable);
  
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_connmode_rq, 
		"direct", 
		direct);
  
  return 0;
}

int8_t 
ipmi_set_serial_connmode (u_int16_t sms_io_base, 
			  u_int8_t channel_number, 
			  u_int8_t basic_mode_enable,
			  u_int8_t ppp_mode_enable,
			  u_int8_t terminal_mode_enable,
			  u_int8_t direct,
			  fiid_obj_t obj_data_rs)
{
  fiid_obj_t obj_data_rq; 
  int8_t status;
  
  obj_data_rq = fiid_obj_alloc (tmpl_set_serial_conf_param_connmode_rq);
  fill_set_serial_connmode (obj_data_rq, 
			    channel_number, 
			    basic_mode_enable, 
			    ppp_mode_enable, 
			    terminal_mode_enable, 
			    direct);
  status = ipmi_kcs_cmd (sms_io_base, IPMI_BMC_IPMB_LUN_BMC, IPMI_NET_FN_TRANSPORT_RQ, 
			 obj_data_rq, tmpl_set_serial_conf_param_connmode_rq, 
			 obj_data_rs, tmpl_set_serial_conf_param_rs);
  free (obj_data_rq);
  return status;
}

int8_t 
fill_set_serial_page_blackout_interval (fiid_obj_t obj_data_rq, 
					u_int8_t channel_number, 
					u_int8_t page_blackout_interval)
{
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_pageblackout_rq, 
		"cmd", 
		IPMI_CMD_SET_SERIAL_MODEM_CONF_PARAM);
  
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_pageblackout_rq, 
		"channel_number", 
		channel_number);
  
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_pageblackout_rq, 
		"parameter_selector", 
		IPMI_SERIAL_PARAM_PAGE_BLACKOUT_INTERVAL);
  
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_pageblackout_rq, 
		"page_blackout_interval", 
		page_blackout_interval);
  
  return 0;
}

int8_t 
ipmi_set_serial_page_blackout_interval (u_int16_t sms_io_base, 
					u_int8_t channel_number, 
					u_int8_t page_blackout_interval, 
					fiid_obj_t obj_data_rs)
{
  fiid_obj_t obj_data_rq; 
  int8_t status;
  
  obj_data_rq = fiid_obj_alloc (tmpl_set_serial_conf_param_pageblackout_rq);
  fill_set_serial_page_blackout_interval (obj_data_rq, 
					  channel_number, 
					  page_blackout_interval);
  status = ipmi_kcs_cmd (sms_io_base, IPMI_BMC_IPMB_LUN_BMC, IPMI_NET_FN_TRANSPORT_RQ, 
			 obj_data_rq, tmpl_set_serial_conf_param_pageblackout_rq, 
			 obj_data_rs, tmpl_set_serial_conf_param_rs);
  free (obj_data_rq);
  return status;
}

int8_t 
fill_set_serial_retry_time (fiid_obj_t obj_data_rq, 
			    u_int8_t channel_number, 
			    u_int8_t retry_time)
{
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_retry_rq, 
		"cmd", 
		IPMI_CMD_SET_SERIAL_MODEM_CONF_PARAM);
    
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_retry_rq, 
		"channel_number", 
		channel_number);
    
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_retry_rq, 
		"parameter_selector", 
		IPMI_SERIAL_PARAM_RETRY_TIME);
    
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_retry_rq, 
		"retry_time", 
		retry_time);
  
  return 0;
}	    

int8_t 
ipmi_set_serial_retry_time (u_int16_t sms_io_base, 
			    u_int8_t channel_number, 
			    u_int8_t retry_time, 
			    fiid_obj_t obj_data_rs)
{
  fiid_obj_t obj_data_rq; 
  int8_t status;
  
  obj_data_rq = fiid_obj_alloc (tmpl_set_serial_conf_param_retry_rq);
  fill_set_serial_retry_time (obj_data_rq, 
			      channel_number, 
			      retry_time);
  status = ipmi_kcs_cmd (sms_io_base, IPMI_BMC_IPMB_LUN_BMC, IPMI_NET_FN_TRANSPORT_RQ, 
			 obj_data_rq, tmpl_set_serial_conf_param_retry_rq, 
			 obj_data_rs, tmpl_set_serial_conf_param_rs);
  free (obj_data_rq);
  return status;
}

int8_t 
fill_set_serial_comm_bits (fiid_obj_t obj_data_rq, 
			   u_int8_t channel_number, 
                           u_int8_t dtr_hangup,
                           u_int8_t flow_control,
                           u_int8_t bit_rate)
{
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_commbits_rq, 
		"cmd", 
		IPMI_CMD_SET_SERIAL_MODEM_CONF_PARAM);
  
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_commbits_rq, 
		"channel_number", 
		channel_number);
  
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_commbits_rq, 
		"parameter_selector", 
		IPMI_SERIAL_PARAM_COMM_BITS);
  
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_commbits_rq, 
		"dtr_hangup", 
		dtr_hangup);
  
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_commbits_rq, 
		"flow_control", 
		flow_control);
  
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_set_serial_conf_param_commbits_rq, 
		"bit_rate", 
		bit_rate);
  
  return 0;
}

int8_t 
ipmi_set_serial_comm_bits (u_int16_t sms_io_base, 
			   u_int8_t channel_number, 
                           u_int8_t dtr_hangup,
                           u_int8_t flow_control,
                           u_int8_t bit_rate,
			   fiid_obj_t obj_data_rs)
{
  fiid_obj_t obj_data_rq; 
  int8_t status;
  
  obj_data_rq = fiid_obj_alloc (tmpl_set_serial_conf_param_retry_rq);
  fill_set_serial_comm_bits (obj_data_rq, 
			     channel_number, 
			     dtr_hangup,
			     flow_control,
			     bit_rate);
  status = ipmi_kcs_cmd (sms_io_base, IPMI_BMC_IPMB_LUN_BMC, IPMI_NET_FN_TRANSPORT_RQ, 
			 obj_data_rq, tmpl_set_serial_conf_param_commbits_rq, 
			 obj_data_rs, tmpl_set_serial_conf_param_rs);
  free (obj_data_rq);
  return status;
}

int8_t 
fill_get_serial_conf_param (fiid_obj_t obj_data_rq, 
			    u_int8_t parameter_selector, 
			    u_int8_t channel_number,
			    u_int8_t parameter_type,
			    u_int8_t set_selector,
			    u_int8_t block_selector)
{
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_get_serial_conf_param_rq, 
		"cmd", 
		IPMI_CMD_GET_SERIAL_MODEM_CONF_PARAM);
    
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_get_serial_conf_param_rq, 
		"channel_number", 
		channel_number);
    
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_get_serial_conf_param_rq, 
		"parameter_type", 
		parameter_type);
    
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_get_serial_conf_param_rq, 
		"parameter_selector", 
		parameter_selector);
    
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_get_serial_conf_param_rq, 
		"set_selector", 
		set_selector);
    
  FIID_OBJ_SET (obj_data_rq, 
		tmpl_get_serial_conf_param_rq, 
		"block_selector", 
		block_selector);
  
  return 0;
}

int8_t 
ipmi_get_serial_connmode (u_int16_t sms_io_base,
                          u_int8_t channel_number,
                          u_int8_t parameter_type,
                          u_int8_t set_selector,
                          u_int8_t block_selector,
                          fiid_obj_t obj_data_rs)
{
  fiid_obj_t obj_data_rq; 
  int8_t status;
  
  obj_data_rq = fiid_obj_alloc (tmpl_get_serial_conf_param_rq);
  fill_get_serial_conf_param (obj_data_rq, 
			      IPMI_SERIAL_PARAM_CONNECTION_MODE, 
			      channel_number, 
			      parameter_type, 
			      set_selector, 
			      block_selector);
  status = ipmi_kcs_cmd (sms_io_base, IPMI_BMC_IPMB_LUN_BMC, IPMI_NET_FN_TRANSPORT_RQ, 
			 obj_data_rq, tmpl_get_serial_conf_param_rq, 
			 obj_data_rs, tmpl_get_serial_conf_param_connmode_rs);
  free (obj_data_rq);
  return status;
}

int8_t 
ipmi_get_serial_page_blackout (u_int16_t sms_io_base,
			       u_int8_t channel_number,
			       u_int8_t parameter_type,
			       u_int8_t set_selector,
			       u_int8_t block_selector,
			       fiid_obj_t obj_data_rs)
{
  fiid_obj_t obj_data_rq; 
  int8_t status;
  
  obj_data_rq = fiid_obj_alloc (tmpl_get_serial_conf_param_rq);
  fill_get_serial_conf_param (obj_data_rq, 
			      IPMI_SERIAL_PARAM_PAGE_BLACKOUT_INTERVAL, 
			      channel_number, 
			      parameter_type, 
			      set_selector, 
			      block_selector);
  status = ipmi_kcs_cmd (sms_io_base, IPMI_BMC_IPMB_LUN_BMC, IPMI_NET_FN_TRANSPORT_RQ, 
			 obj_data_rq, tmpl_get_serial_conf_param_rq, 
			 obj_data_rs, tmpl_get_serial_conf_param_pageblackout_rs);
  free (obj_data_rq);
  return status;
}

int8_t 
ipmi_get_serial_retry_time (u_int16_t sms_io_base,
			    u_int8_t channel_number,
			    u_int8_t parameter_type,
			    u_int8_t set_selector,
			    u_int8_t block_selector,
			    fiid_obj_t obj_data_rs)
{
  fiid_obj_t obj_data_rq; 
  int8_t status;
  
  obj_data_rq = fiid_obj_alloc (tmpl_get_serial_conf_param_rq);
  fill_get_serial_conf_param (obj_data_rq, 
			      IPMI_SERIAL_PARAM_RETRY_TIME, 
			      channel_number, 
			      parameter_type, 
			      set_selector, 
			      block_selector);
  status = ipmi_kcs_cmd (sms_io_base, IPMI_BMC_IPMB_LUN_BMC, IPMI_NET_FN_TRANSPORT_RQ, 
			 obj_data_rq, tmpl_get_serial_conf_param_rq, 
			 obj_data_rs, tmpl_get_serial_conf_param_retry_rs);
  free (obj_data_rq);
  return status;
}

int8_t 
ipmi_get_serial_comm_bits (u_int16_t sms_io_base,
			   u_int8_t channel_number,
			   u_int8_t parameter_type,
			   u_int8_t set_selector,
			   u_int8_t block_selector,
			   fiid_obj_t obj_data_rs)
{
  fiid_obj_t obj_data_rq; 
  int8_t status;
  
  obj_data_rq = fiid_obj_alloc (tmpl_get_serial_conf_param_rq);
  fill_get_serial_conf_param (obj_data_rq, 
			      IPMI_SERIAL_PARAM_COMM_BITS, 
			      channel_number, 
			      parameter_type, 
			      set_selector, 
			      block_selector);
  status = ipmi_kcs_cmd (sms_io_base, IPMI_BMC_IPMB_LUN_BMC, IPMI_NET_FN_TRANSPORT_RQ, 
			 obj_data_rq, tmpl_get_serial_conf_param_rq, 
			 obj_data_rs, tmpl_get_serial_conf_param_commbits_rs);
  free (obj_data_rq);
  return status;
}

